@@ -2,6 +2,15 @@
 ;;; Commentary: Prepare to setup Eask environment for sandboxing
 ;;; Code:
 
+;;
+;;; Requirement
+
+(when (version< emacs-version "26.1")
+  (error "Eask requires Emacs 26.1 and above!"))
+
+;;
+;;; Includes
+
 (require 'ansi-color)
 (require 'package)
 (require 'project)
@@ -33,7 +42,7 @@
 (declare-function ansi-white "ext:ansi.el")
 
 ;;
-;;; Environments
+;;; Environment
 
 ;; Determine the underlying operating system
 (defconst eask-is-windows (memq system-type '(cygwin windows-nt ms-dos))
@@ -59,6 +68,9 @@
       package-archives           nil            ; Leave it to custom use
       package-archive-priorities nil)
 
+(defvar eask-dot-emacs-file nil
+  "Variable hold .emacs file location.")
+
 (defun eask--load--adv (fnc &rest args)
   "Prevent `_prepare.el' loading twice.
 
@@ -67,16 +79,37 @@ Arguments FNC and ARGS are used for advice `:around'."
 (advice-add 'load :around #'eask--load--adv)
 
 (defconst eask-has-colors (getenv "EASK_HASCOLORS")
-  "Return non-nil if terminal support colors.")
+  "Return non-nil if terminal supports colors.")
 
 (defconst eask-homedir (getenv "EASK_HOMEDIR")
-  "Eask temporary storage.")
+  "Eask's home directory path.")
 
 (defconst eask-invocation (getenv "EASK_INVOCATION")
-  "Eask invocation program.")
+  "Eask's invocation program path.")
 
 (defconst eask-is-pkg (getenv "EASK_IS_PKG")
-  "Eask is pkg.")
+  "Return non-nil if Eask is packaged.")
+
+(defconst eask-rest
+  (let ((args (getenv "EASK_REST_ARGS")))
+    (setq args (ignore-errors (split-string args ",")))
+    args)
+  "Eask's arguments after command separator `--'; return a list.
+
+If the argument is `-- arg0 arg1'; it will return `(arg0 arg1)'.")
+
+(defun eask-rest ()
+  "Eask's arguments after command separator `--'; return a string.
+
+If the argument is `-- arg0 arg1'; it will return `arg0 arg1'."
+  (mapconcat #'identity eask-rest " "))
+
+(defcustom eask-import-timeout 10
+  "Number of seconds before timing out elisp importation attempts.
+If nil, never time out."
+  :type '(choice (number :tag "Number of seconds")
+                 (const  :tag "Never time out" nil))
+  :group 'eask)
 
 ;;
 ;;; Execution
@@ -119,16 +152,34 @@ will return `lint/checkdoc' with a dash between two subcommands."
                                     (list script-file))
                  "/"))))
 
+(defun eask-command-p (commands)
+  "Return t if COMMANDS is the current command."
+  (member (eask-command) (eask-listify commands)))
+
 (defun eask-special-p ()
-  "Return t if the command that can be run without Eask-file existence."
-  (member (eask-command) '("init/cask" "init/keg"
-                           "init/source"
-                           "cat" "keywords"
-                           "generate/ignore" "generate/license")))
+  "Return t if the command that can be run without Eask-file existence.
+
+These commands will first respect the current workspace.  If the current
+workspace has no valid Eask-file; it will load global workspace instead."
+  (eask-command-p '("init" "init/source" "init/cask" "init/eldev" "init/keg"
+                    "create/package" "create/elpa"
+                    "bump" "cat" "keywords" "repl"
+                    "generate/ignore" "generate/license"
+                    "test/melpazoid")))
+
+(defun eask-execution-p ()
+  "Return t if the command is the execution command.
+
+This is added because we don't want to pollute `error' and `warn' functions."
+  (eask-command-p '("load" "exec" "emacs" "eval" "repl"
+                    "run/script" "run/command")))
 
 (defun eask-checker-p ()
-  "Return t if running Eask as the checker."
-  (member (eask-command) '("check-eask")))
+  "Return t if running Eask as the checker.
+
+Without this flag, the process will be terminated once the error is occurred.
+This flag allows you to run through operations without reporting errors."
+  (eask-command-p '("analyze")))
 
 (defun eask-script (script)
   "Return full SCRIPT filename."
@@ -149,6 +200,15 @@ the `eask-start' execution.")
       (load script-file nil t)
     (eask-error "Script missing %s" script-file)))
 
+(defun eask-import (url)
+  "Load and eval the script from a URL."
+  (with-current-buffer (url-retrieve-synchronously url t nil eask-import-timeout)
+    (goto-char (point-min))
+    (re-search-forward "^$" nil 'move)
+    (forward-char)
+    (delete-region (point-min) (point))
+    (eval-buffer)))
+
 ;;
 ;;; Util
 
@@ -173,6 +233,14 @@ Argument BODY are forms for execution."
   "Convert OBJ to string."
   (format "%s" obj))
 
+(defun eask-2url (url)
+  "Convert secure/insecure URL."
+  (if (and url
+           (gnutls-available-p)
+           (eask-network-insecure-p))
+      (eask-s-replace "https://" "http://" url)
+    url))
+
 (defun eask-listify (obj)
   "Turn OBJ to list."
   (if (listp obj) obj (list obj)))
@@ -216,11 +284,29 @@ The function `directory-empty-p' only exists 28.1 or above; copied it."
          ;; 27.2 or lower!
          (null (directory-files dir nil directory-files-no-dot-files-regexp t)))))
 
-(defun eask-guess-entry-point (project-name)
-  "Return the guess entry point by its PROJECT-NAME."
-  (if (string-suffix-p ".el" project-name)
-      project-name
-    (format "%s.el" project-name)))
+(defun eask--guess-package-name (basename)
+  "Convert the BASENAME to a valid, commonly seen package name."
+  (when-let ((name (ignore-errors (downcase basename))))
+    (setq name (eask-s-replace "emacs-" "" name)
+          name (eask-s-replace "-emacs" "" name)
+          name (replace-regexp-in-string "[.-]el$" "" name))
+    name))
+
+(defun eask-guess-package-name (&optional basename)
+  "Return the possible package name.
+
+Optional argument BASENAME accepts a string; it will convert the string to a
+valid, commonly seen package name."
+  (or (eask--guess-package-name basename)
+      (eask-package-name)
+      (eask--guess-package-name
+       (ignore-errors (file-name-nondirectory
+                       (file-name-sans-extension eask-package-file))))))
+
+(defun eask-guess-entry-point (&optional basename)
+  "Return the guess entry point by its BASENAME."
+  (let ((name (eask-guess-package-name basename)))
+    (format "%s.el" name)))
 
 (defun eask-read-string (prompt &optional
                                 initial-input
@@ -234,6 +320,28 @@ and INHERIT-INPUT-METHOD see function `read-string' for more information."
   (let ((str (read-string prompt initial-input history default-value inherit-input-method)))
     (eask-s-replace "\"" "" str)))
 
+(defun eask--goto-line (line)
+  "Go to LINE."
+  (goto-char (point-min))
+  (forward-line (1- line)))
+
+(defun eask--column-at-point (point)
+  "Get column at POINT."
+  (save-excursion (goto-char point) (current-column)))
+
+(defconst eask-buffer-name "*eask*"
+  "Buffer name is used for temporary storage throughout the life cycle.")
+
+(defmacro eask-with-buffer (&rest body)
+  "Create a temporary buffer (for this program), and evaluate BODY there."
+  (declare (indent 0) (debug t))
+  `(with-current-buffer (get-buffer-create ,eask-buffer-name) ,@body))
+
+(defmacro eask-with-temp-buffer (&rest body)
+  "Create a temporary buffer (for this program), and evaluate BODY there."
+  (declare (indent 0) (debug t))
+  `(eask-with-buffer (erase-buffer) ,@body))
+
 ;;
 ;;; Progress
 
@@ -389,6 +497,21 @@ Arguments FNC and ARGS are used for advice `:around'."
 ;;
 ;;; Package
 
+(defun eask-use-legacy-package-build ()
+  "Return t if using the legacy `package-build' package."
+  (version< emacs-version "27.1"))
+
+(defun eask-load-legacy-package-build ()
+  "Load the legacy `package-build' package."
+  (when (eask-use-legacy-package-build)
+    (add-to-list 'load-path
+                 (format "%sextern/package-build/%s/"
+                         eask-lisp-root
+                         emacs-major-version)
+                 t)))
+
+(eask-load-legacy-package-build)
+
 (defun eask--update-exec-path ()
   "Add all bin directory to the variable `exec-path'."
   (dolist (entry (directory-files package-user-dir t directory-files-no-dot-files-regexp))
@@ -443,7 +566,7 @@ scope of the dependencies (it's either `production' or `development')."
   (eask-defvc< 27 (eask-pkg-init))  ; XXX: remove this after we drop 26.x
   (when eask-depends-on-recipe-p
     (eask-log "Installing required external packages...")
-    (eask-with-archives "melpa"
+    (eask-with-archives '("gnu" "melpa")
       (eask-package-install 'package-build))
     (eask-with-progress
       "Building temporary archives (this may take a while)... "
@@ -546,7 +669,7 @@ Argument BODY are forms for execution."
      ((progn
         (eask-pkg-init)
         (unless (eask-package-installable-p pkg)
-          (eask-error "Package not installable `%s'; make sure package archive is included" pkg))))
+          (eask-error "Package not installable `%s'; make sure the package archive (source) is included" pkg))))
      ((when-let* ((desc (eask-package-desc pkg))
                   (req-emacs (assoc 'emacs (package-desc-reqs desc)))
                   (req-emacs (package-version-join (nth 0 (cdr req-emacs))))
@@ -554,10 +677,10 @@ Argument BODY are forms for execution."
         (if (eask-strict-p)
             (eask-error "  - %sSkipping %s (%s)... it requires Emacs %s and above ✗"
                         eask--action-prefix
-                        pkg (eask-package--version-string pkg) emacs-version)
+                        pkg (eask-package--version-string pkg) req-emacs)
           (eask-msg "  - %sSkipping %s (%s)... it requires Emacs %s and above ✗"
                     eask--action-prefix
-                    name version (ansi-yellow emacs-version)))))
+                    name version (ansi-yellow req-emacs)))))
      (t
       (eask--pkg-process pkg
         (eask-with-progress
@@ -718,6 +841,9 @@ full detials."
 (defun eask-number-p ()
   "Non-nil when flag is on (`-n', `--number')."
   (eask--flag "--number"))
+(defun eask-yes-p ()
+  "Non-nil when flag is on (`--yes')."
+  (eask--flag "--yes"))
 
 ;;; String (with arguments)
 (defun eask-output ()
@@ -806,7 +932,8 @@ other scripts internally.  See function `eask-call'.")
      "--no-color"
      "--clean"
      "--json"
-     "--number"))
+     "--number"
+     "--yes"))
   "List of boolean type options.")
 
 (defconst eask--option-args
@@ -837,8 +964,10 @@ Simply remove `--eask' for each option, like `--eask--strict' to `--strict'."
             (eask-s-replace "--eask" "" arg))
           eask-argv))
 
-(defun eask-args ()
-  "Get all arguments except options."
+(defun eask-args (&optional index)
+  "Get all arguments except options
+
+If the optional argument INDEX is non-nil, return the element."
   (let ((argv (cl-remove-if (lambda (arg) (member arg eask--option-switches)) eask-argv))
         (args) (skip-next))
     (dolist (arg argv)
@@ -846,7 +975,8 @@ Simply remove `--eask' for each option, like `--eask--strict' to `--strict'."
         (if (member arg eask--option-args)
             (setq skip-next t)
           (push arg args))))
-    (reverse args)))
+    (setq args (reverse args))
+    (if index (nth 0 args) args)))
 
 (defmacro eask--batch-mode (&rest body)
   "Execute forms BODY in batch-mode."
@@ -859,14 +989,7 @@ Simply remove `--eask' for each option, like `--eask--strict' to `--strict'."
   "Execute BODY with workspace setup."
   (declare (indent 0) (debug t))
   `(eask--batch-mode
-     (let (;; XXX: this will make command `info', `files' work as expected;
-           ;; but the relative paths file spec will be lost...
-           ;;
-           ;; So commands like `load' would NOT work!
-           (default-directory (cond ((eask-global-p) eask-homedir)
-                                    ((eask-config-p) user-emacs-directory)
-                                    (t default-directory)))
-           (alist))
+     (let ((alist))
        (dolist (cmd eask--command-list)
          (push (cons cmd (lambda (&rest _))) alist))
        (setq command-switch-alist (append command-switch-alist alist))
@@ -914,6 +1037,12 @@ Argument BODY are forms for execution."
         (defalias keyword (symbol-function old))))
      result))
 
+(defun eask-working-directory ()
+  "Return the working directory of the program going to be executed."
+  (cond ((eask-config-p) user-emacs-directory)
+        ((eask-global-p) (expand-file-name "../../" user-emacs-directory))
+        (t default-directory)))
+
 (defvar eask-file nil "The Eask file's filename.")
 (defvar eask-file-root nil "The Eask file's directory.")
 
@@ -981,7 +1110,7 @@ This uses function `locate-dominating-file' to look up directory tree."
   "Try load the Eask-file in START-PATH."
   (when-let* ((files (eask--find-files start-path))
               (file (car files)))
-    (eask-file-load file)))
+    (eask--unsilent (eask-file-load file))))
 
 (defmacro eask--with-hooks (&rest body)
   "Execute BODY with before/after hooks."
@@ -989,11 +1118,12 @@ This uses function `locate-dominating-file' to look up directory tree."
   `(let* ((command (eask-command))
           (before  (concat "eask-before-" command "-hook"))
           (after   (concat "eask-after-" command "-hook")))
-     (run-hooks 'eask-before-command-hook)
-     (run-hooks (intern before))
-     ,@body
-     (run-hooks (intern after))
-     (run-hooks 'eask-after-command-hook)))
+     (eask--unsilent
+       (run-hooks 'eask-before-command-hook)
+       (run-hooks (intern before))
+       ,@body
+       (run-hooks (intern after))
+       (run-hooks 'eask-after-command-hook))))
 
 (defmacro eask--setup-home (dir &rest body)
   "Set up config directory in DIR, then execute BODY."
@@ -1001,10 +1131,26 @@ This uses function `locate-dominating-file' to look up directory tree."
   `(let* ((user-emacs-directory (expand-file-name (concat ".eask/" emacs-version "/") ,dir))
           (package-user-dir (expand-file-name "elpa" user-emacs-directory))
           (early-init-file (locate-user-emacs-file "early-init.el"))
+          (eask-dot-emacs-file (locate-user-emacs-file ".emacs"))
           (user-init-file (locate-user-emacs-file "init.el"))
           (custom-file (locate-user-emacs-file "custom.el")))
      ,@body))
 
+;; NOTE: If you modified this function, make sure you modified `core/emacs.el'
+;; file as well!
+(defun eask--load-config ()
+  "Load configuration if valid."
+  (let ((inhibit-config (eask-quick-p)))
+    (eask-with-progress
+      (ansi-green "Loading configuration... ")
+      (eask-with-verbosity 'all
+        (unless inhibit-config
+          (when (version<= "27" emacs-version)
+            (load early-init-file t))
+          (load eask-dot-emacs-file t)
+          (load user-init-file t)))
+      (ansi-green (if inhibit-config "skipped ✗" "done ✓")))))
+
 (defmacro eask-start (&rest body)
   "Execute BODY with workspace setup."
   (declare (indent 0) (debug t))
@@ -1014,52 +1160,68 @@ This uses function `locate-dominating-file' to look up directory tree."
        (eask--setup-env
          (eask--handle-global-options)
          (cond
-          ((or (eask-global-p) (eask-special-p))  ; Commands without Eask-file needed!
+          ((eask-config-p)
+           (let ((early-init-file (locate-user-emacs-file "early-init.el"))
+                 (eask-dot-emacs-file (locate-user-emacs-file "../.emacs"))
+                 (user-init-file (locate-user-emacs-file "init.el")))
+             ;; We accept Eask-file in `config' scope, but it shouldn't be used
+             ;; for the sandbox.
+             (eask-with-verbosity 'debug
+               (if (eask-file-try-load user-emacs-directory)
+                   (eask-msg "✓ Loading config Eask file in %s... done!" eask-file)
+                 (eask-msg "✗ Loading config Eask file... missing!"))
+               (eask-msg ""))
+             (package-activate-all)
+             (eask--load-config)
+             (eask--with-hooks ,@body)))
+          ((eask-global-p)
            (eask--setup-home (concat eask-homedir "../")  ; `/home/user/', escape `.eask'
              (let ((eask--first-init-p (not (file-directory-p user-emacs-directory))))
                ;; We accept Eask-file in `global' scope, but it shouldn't be used
                ;; for the sandbox.
                (eask-with-verbosity 'debug
-                 (eask-ignore-errors  ; Again, without Eask-file needed!
-                   (if (eask-file-try-load "./")
+                 (eask-ignore-errors  ; Eask-file is optional!
+                   (if (eask-file-try-load eask-homedir)
                        (eask-msg "✓ Loading global Eask file in %s... done!" eask-file)
                      (eask-msg "✗ Loading global Eask file... missing!")))
                  (eask-msg ""))
                (package-activate-all)
                (ignore-errors (make-directory package-user-dir t))
+               (eask-with-verbosity 'debug (eask--load-config))
+               (eask--with-hooks ,@body))))
+          ((eask-special-p)  ; Commands without Eask-file needed!
+           ;; First, try to find a valid Eask-file!
+           (eask-file-try-load default-directory)
+           ;; Then setup the user directory according to the Eask-file!
+           (eask--setup-home (or eask-file-root
+                                 (concat eask-homedir "../"))
+             (let ((eask--first-init-p (not (file-directory-p user-emacs-directory)))
+                   (scope (if eask-file-root "" "global ")))
+               (eask-with-verbosity 'debug
+                 (eask-ignore-errors  ; Again, without Eask-file needed!
+                   (if (or eask-file-root
+                           (eask-file-try-load eask-homedir))
+                       (eask-msg "✓ Loading %sEask file in %s... done!" scope eask-file)
+                     (eask-msg "✗ Loading %sEask file... missing!" scope)))
+                 (eask-msg ""))
+               (package-activate-all)
+               (ignore-errors (make-directory package-user-dir t))
+               (eask-with-verbosity 'debug (eask--load-config))
                (eask--with-hooks ,@body))))
-          ((eask-config-p)
-           (let ((inhibit-config (eask-quick-p)))
-             ;; We accept Eask-file in `config' scope, but it shouldn't be used
-             ;; for the sandbox.
-             (eask-with-verbosity 'debug
-               (if (eask-file-try-load "./")
-                   (eask-msg "✓ Loading config Eask file in %s... done!" eask-file)
-                 (eask-msg "✗ Loading config Eask file... missing!"))
-               (eask-msg ""))
-             (package-activate-all)
-             (eask-with-progress
-               (ansi-green "Loading your configuration... ")
-               (eask-with-verbosity 'all
-                 (unless inhibit-config
-                   (load (locate-user-emacs-file "early-init.el") t)
-                   (load (locate-user-emacs-file "../.emacs") t)
-                   (load (locate-user-emacs-file "init.el") t)))
-               (ansi-green (if inhibit-config "skipped ✗" "done ✓")))
-             (eask--with-hooks ,@body)))
           (t
            (eask--setup-home nil  ; `nil' is the `default-directory'
              (let ((eask--first-init-p (not (file-directory-p user-emacs-directory))))
                (eask-with-verbosity 'debug
-                 (if (eask-file-try-load "./")
+                 (if (eask-file-try-load default-directory)
                      (eask-msg "✓ Loading Eask file in %s... done!" eask-file)
                    (eask-msg "✗ Loading Eask file... missing!"))
-                 (message ""))
+                 (eask-msg ""))
                (if (not eask-file)
                    (eask-help "core/init")
                  (package-activate-all)
                  (ignore-errors (make-directory package-user-dir t))
                  (eask--silent (eask-setup-paths))
+                 (eask-with-verbosity 'debug (eask--load-config))
                  (eask--with-hooks ,@body))))))))))
 
 ;;
@@ -1085,6 +1247,12 @@ This uses function `locate-dominating-file' to look up directory tree."
     (nongnu-devel . "https://elpa.nongnu.org/nongnu-devel/"))
   "Mapping of source name and url.")
 
+(defun eask-source-url (name &optional location)
+  "Get the source url by it's NAME and LOCATION."
+  (setq location (or location (cdr (assq (intern (eask-2str name)) eask-source-mapping)))
+        location (eask-2url location))
+  location)
+
 (defvar eask-package            nil)
 (defvar eask-package-desc       nil)  ; package descriptor
 (defvar eask-package-descriptor nil)
@@ -1202,13 +1370,14 @@ version number.  DESCRIPTION is the package description."
                     ((eask-pkg-el)                     ; if -pkg.el is presented,
                      (setq skipped t) nil)             ; skip it
                     (t (package-buffer-info))))))      ; default read main package file
-    (eask-msg (concat
-               (if eask-package-desc "✓ " "✗ ")
-               "Try constructing the package-descriptor (%s)... "
-               (cond (eask-package-desc "succeeded!")
-                     (skipped "skipped!")
-                     (t "failed!")))
-              (file-name-nondirectory file))))
+    (eask-with-verbosity 'debug
+      (eask-msg (concat
+                 (if eask-package-desc "✓ " "✗ ")
+                 "Try constructing the package-descriptor (%s)... "
+                 (cond (eask-package-desc "succeeded!")
+                       (skipped           "skipped!")
+                       (t                 "failed!")))
+                (file-name-nondirectory file)))))
 
 (defun eask-f-package-file (file)
   "Set package FILE."
@@ -1266,12 +1435,8 @@ argument COMMAND."
   (when (symbolp name) (setq name (eask-2str name)))  ; ensure to string, accept symbol
   (when (assoc name package-archives)
     (eask-error "Multiple definition of source `%s'" name))
-  (setq location (or location (cdr (assq (intern name) eask-source-mapping))))
+  (setq location (eask-source-url name location))
   (unless location (eask-error "Unknown package archive `%s'" name))
-  (when (and location
-             (gnutls-available-p)
-             (not (eask-network-insecure-p)))
-    (setq location (eask-s-replace "https://" "http://" location)))
   (add-to-list 'package-archives (cons name location) t))
 
 (defun eask-f-source-priority (archive-id &optional priority)
@@ -1394,7 +1559,7 @@ Standard is, 0 (error), 1 (warning), 2 (info), 3 (log), 4 (debug), 5 (all)."
     (`error 0)
     (t symbol)))
 
-(defun eask--reach-verbosity-p (symbol)
+(defun eask-reach-verbosity-p (symbol)
   "Return t if SYMBOL reach verbosity (should be printed)."
   (>= eask-verbosity (eask--verb2lvl symbol)))
 
@@ -1403,7 +1568,7 @@ Standard is, 0 (error), 1 (warning), 2 (info), 3 (log), 4 (debug), 5 (all)."
 
 Execute forms BODY limit by the verbosity level (SYMBOL)."
   (declare (indent 1) (debug t))
-  `(if (eask--reach-verbosity-p ,symbol) (progn ,@body)
+  `(if (eask-reach-verbosity-p ,symbol) (progn ,@body)
      (eask--silent ,@body)))
 
 (defmacro eask-with-verbosity-override (symbol &rest body)
@@ -1411,7 +1576,7 @@ Execute forms BODY limit by the verbosity level (SYMBOL)."
 
 Execute forms BODY limit by the verbosity level (SYMBOL)."
   (declare (indent 1) (debug t))
-  `(if (eask--reach-verbosity-p ,symbol) (eask--unsilent ,@body)
+  `(if (eask-reach-verbosity-p ,symbol) (eask--unsilent ,@body)
      (eask--silent ,@body)))
 
 (defun eask--ansi (symbol string)
@@ -1489,20 +1654,41 @@ character."
          (string (eask--msg-displayable-kwds string)))
     string))
 
-(defun eask-msg (msg &rest args)
-  "Like function `message' but replace unicodes with color.
+(defun eask-princ (object &optional stderr)
+  "Like function `princ'; with flag STDERR.
+
+For argument OBJECT, please see function `princ' for the detials.
+
+If optional argument STDERR is non-nil; use stderr instead."
+  (unless inhibit-message
+    (princ object (when stderr #'external-debugging-output))))
+
+(defun eask-print (msg &rest args)
+  "Standard output printing without newline.
 
 For arguments MSG and ARGS, please see function `eask--format-paint-kwds' for
 the detials."
-  (message (apply #'eask--format-paint-kwds msg args)))
+  (eask-princ (apply #'eask--format-paint-kwds msg args)))
+
+(defun eask-println (msg &rest args)
+  "Like the function `eask-print' but contains the newline at the end.
+
+For arguments MSG and ARGS, please see function `eask-print' for the detials."
+  (apply #'eask-print (concat msg "\n") args))
+
+(defun eask-msg (msg &rest args)
+  "Like the function `message' but replace unicode with color.
+
+For arguments MSG and ARGS, please see function `eask--format-paint-kwds' for
+the detials."
+  (apply #'eask-write msg args)
+  (eask-princ "\n" t))
 
 (defun eask-write (msg &rest args)
-  "Like function `eask-msg' but without newline at the end.
+  "Like the function `eask-msg' but without newline at the end.
 
-For arguments MSG and ARGS, please see function `eask--format-paint-kwds' for
-the detials."
-  (unless inhibit-message
-    (princ (apply #'eask--format-paint-kwds msg args) 'external-debugging-output)))
+For arguments MSG and ARGS, please see function `eask-msg' for the detials."
+  (eask-princ (apply #'eask--format-paint-kwds msg args) t))
 
 (defun eask-report (&rest args)
   "Report error/warning depends on strict flag.
@@ -1555,8 +1741,6 @@ Arguments FNC and ARGS are used for advice `:around'."
     (eask--trigger-error))
   (when debug-on-error (apply fnc args)))
 
-(advice-add 'error :around #'eask--error)
-
 (defun eask--warn (fnc &rest args)
   "On warn.
 
@@ -1567,7 +1751,10 @@ Arguments FNC and ARGS are used for advice `:around'."
     (run-hook-with-args 'eask-on-warning-hook 'warn msg))
   (eask--silent (apply fnc args)))
 
-(advice-add 'warn :around #'eask--warn)
+;; Don't pollute outer exection.
+(unless (eask-execution-p)
+  (advice-add 'error :around #'eask--error)
+  (advice-add 'warn :around #'eask--warn))
 
 ;;
 ;;; Log
@@ -1600,12 +1787,6 @@ Arguments FNC and ARGS are used for advice `:around'."
 ;;
 ;;; File
 
-(defun eask-guess-package-name ()
-  "Return the possible package name."
-  (or (eask-package-name)
-      (ignore-errors (file-name-nondirectory
-                      (file-name-sans-extension eask-package-file)))))
-
 (defun eask-files-spec ()
   "Return files spec."
   (or eask-files package-build-default-files-spec))
@@ -1807,7 +1988,17 @@ variable we use to test validation."
   :group 'eask)
 
 (defcustom eask-dist-path "dist"
-  "Name of default target directory for building packages."
+  "Default path where to place the package artifact."
+  :type 'string
+  :group 'eask)
+
+(defcustom eask-docs-path "docs/public/"
+  "Default path where to place the documentation."
+  :type 'string
+  :group 'eask)
+
+(defcustom eask-recipe-path "recipes"
+  "Name of default target directory for placing recipes."
   :type 'string
   :group 'eask)
 
@@ -1823,6 +2014,20 @@ variable we use to test validation."
     (eask-msg ""))
   (setq eask-lint-first-file-p t))
 
+;;
+;;; API
+
+(defvar eask-commands nil
+  "List of defined commands.")
+
+(defmacro eask-defcommand (name &rest body)
+  "Define an Eask command."
+  (declare (doc-string 2) (indent 1))
+  (or name (error "Cannot define '%s' as a command" name))
+  (push name eask-commands)
+  (setq eask-commands (delete-dups eask-commands))
+  `(defun ,name nil ,@body))
+
 ;;
 ;;; Externals
 
@@ -1831,10 +2036,4 @@ variable we use to test validation."
 (eask-load "extern/package")
 (eask-load "extern/package-build")
 
-;;
-;;; Requirement
-
-(when (version< emacs-version "26.1")
-  (eask-error "Eask requires Emacs 26.1 and above!"))
-
 ;;; _prepare.el ends here
